/*
 * display.c
 *
 *  Created on: Apr 6, 2012
 *      Author: Philip
 */
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"

#include "lcd.h"
#include "time.h"

void lcd_clear(void){
}

inline void lcd_setRegister(uint16_t regVal, uint16_t datVal){
	LCD_INST_ADD = regVal;
	LCD_DATA_ADD = datVal;
}

inline uint16_t lcd_readRegister(uint16_t regVal){
	uint16_t data;
	LCD_INST_ADD = regVal;
	data = LCD_DATA_ADD;
	return data;
}

void lcd_init(){
	GPIO_InitTypeDef GPIO_InitStructure;

	// enable clocks for used ports
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD | RCC_AHB1Periph_GPIOG | RCC_AHB1Periph_GPIOE | RCC_AHB1Periph_GPIOF, ENABLE);
	RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);

	//Enable LCD reset pin:
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;

	GPIO_InitStructure.GPIO_Pin = LCD_RES_PIN;
	GPIO_Init(LCD_CTRL_PORT, &GPIO_InitStructure);

	// pull reset low during remaining init
	GPIO_ResetBits(LCD_CTRL_PORT, LCD_RES_PIN);

	// init FSMC
	RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource0, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource1, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource8, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource9, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource10, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource7, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource5, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource4, GPIO_AF_FSMC);

	GPIO_PinAFConfig(GPIOE, GPIO_PinSource7, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource8, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource9, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource10, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource11, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource12, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource13, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource14, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource15, GPIO_AF_FSMC);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOE, &GPIO_InitStructure);

	time_wait(100);

	FSMC_NORSRAMInitTypeDef FSMC_NORSRAMInitStructure;
	FSMC_NORSRAMTimingInitTypeDef FSMC_NORSRAMTimingInitStructure;

	FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM1;
	FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
	FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
	FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
	FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
	FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
	FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
	FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;
	FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Enable;//disable

	FSMC_NORSRAMTimingInitStructure.FSMC_AddressSetupTime = 2; //0
	FSMC_NORSRAMTimingInitStructure.FSMC_AddressHoldTime = 3; //0
	FSMC_NORSRAMTimingInitStructure.FSMC_DataSetupTime = 4; //3
	FSMC_NORSRAMTimingInitStructure.FSMC_BusTurnAroundDuration = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_CLKDivision = 1;//1
	FSMC_NORSRAMTimingInitStructure.FSMC_DataLatency = 0;
	FSMC_NORSRAMTimingInitStructure.FSMC_AccessMode = FSMC_AccessMode_A;

	FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &FSMC_NORSRAMTimingInitStructure;
	FSMC_NORSRAMInit( &FSMC_NORSRAMInitStructure);

	FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM1, ENABLE);

	//pull reset low for at least 1ms

	//FROM app notes
	GPIO_SetBits(LCD_CTRL_PORT, LCD_RES_PIN);
	time_wait(100);
	GPIO_ResetBits(LCD_CTRL_PORT, LCD_RES_PIN);
	time_wait(100);
	GPIO_SetBits(LCD_CTRL_PORT, LCD_RES_PIN);

	FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM1, ENABLE);
	time_wait(5);

	// Start Initial Sequence

	lcd_setRegister(0x00E5, 0x78F0); // set SRAM internal timing
	lcd_setRegister(0x0001, 0x0100); // set SS and SM bit
	lcd_setRegister(0x0002, 0x0700); // set 1 line inversion
	lcd_setRegister(0x0003, 0x1030); // set GRAM write direction and BGR=1.
	lcd_setRegister(0x0004, 0x0000); // Resize register
	lcd_setRegister(0x0008, 0x0207); // set the back porch and front porch
	lcd_setRegister(0x0009, 0x0000); // set non-display area refresh cycle ISC[3:0]
	lcd_setRegister(0x000A, 0x0000); // FMARK function
	lcd_setRegister(0x000C, 0x0000); // RGB interface setting
	lcd_setRegister(0x000D, 0x0000); // Frame marker Position
	lcd_setRegister(0x000F, 0x0000); // RGB interface polarity

	// Power On sequence
	lcd_setRegister(0x0010, 0x0000); // SAP, BT[3:0], AP, DSTB, SLP, STB
	lcd_setRegister(0x0011, 0x0007); // DC1[2:0], DC0[2:0], VC[2:0]
	lcd_setRegister(0x0012, 0x0000); // VREG1OUT voltage
	lcd_setRegister(0x0013, 0x0000); // VDV[4:0] for VCOM amplitude
	lcd_setRegister(0x0007, 0x0001);
	time_wait(50); // Dis-charge capacitor power voltage
	lcd_setRegister(0x0010, 0x1090); // 1490//SAP, BT[3:0], AP, DSTB, SLP, STB
	lcd_setRegister(0x0011, 0x0227); // DC1[2:0], DC0[2:0], VC[2:0]
	time_wait(50); // Delay 50ms
	lcd_setRegister(0x0012, 0x001F); //001C// Internal reference voltage= Vci;
	time_wait(50); // Delay 50ms
	lcd_setRegister(0x0013, 0x1500); //0x1000//1400   Set VDV[4:0] for VCOM amplitude  1A00
	lcd_setRegister(0x0029, 0x0027); //0x0012 //001a  Set VCM[5:0] for VCOMH  //0x0025  0034
	lcd_setRegister(0x002B, 0x000D); // Set Frame Rate   000C
	time_wait(50); // Delay 50ms
	lcd_setRegister(0x0020, 0x0000); // GRAM horizontal Address
	lcd_setRegister(0x0021, 0x0000); // GRAM Vertical Address
	// ----------- Adjust the Gamma Curve ----------//
	lcd_setRegister(0x0030, 0x0000);
	lcd_setRegister(0x0031, 0x0707);
	lcd_setRegister(0x0032, 0x0307);
	lcd_setRegister(0x0035, 0x0200);
	lcd_setRegister(0x0036, 0x0008);//0207
	lcd_setRegister(0x0037, 0x0004);//0306
	lcd_setRegister(0x0038, 0x0000);//0102
	lcd_setRegister(0x0039, 0x0707);//0707
	lcd_setRegister(0x003C, 0x0002);//0702
	lcd_setRegister(0x003D, 0x1D04);//1604

	//------------------ Set GRAM area ---------------//
	lcd_setRegister(0x0050, 0x0000); // Horizontal GRAM Start Address
	lcd_setRegister(0x0051, 0x00EF); // Horizontal GRAM End Address
	lcd_setRegister(0x0052, 0x0000); // Vertical GRAM Start Address
	lcd_setRegister(0x0053, 0x013F); // Vertical GRAM Start Address
	lcd_setRegister(0x0060, 0xA700); // Gate Scan Line
	lcd_setRegister(0x0061, 0x0001); // NDL,VLE, REV
	lcd_setRegister(0x006A, 0x0000); // set scrolling line
	//-------------- Partial Display Control ---------//
	lcd_setRegister(0x0080, 0x0000);
	lcd_setRegister(0x0081, 0x0000);
	lcd_setRegister(0x0082, 0x0000);
	lcd_setRegister(0x0083, 0x0000);
	lcd_setRegister(0x0084, 0x0000);
	lcd_setRegister(0x0085, 0x0000);
	//-------------- Panel Control -------------------//
	lcd_setRegister(0x0090, 0x0010);
	lcd_setRegister(0x0092, 0x0600);
	lcd_setRegister(0x0007, 0x0133); // 262K color and display ON

	uint16_t id = lcd_readRegister(0x00);

}

void lcd_setArea(uint8_t x1, uint16_t y1, uint16_t x2, uint16_t y2){

	lcd_setRegister(0x0020, x1); // set gram horizontal start
	lcd_setRegister(0x0021, y1); // set gram vertical start
	lcd_setRegister(0x0050, x1); // set horizontal start
	lcd_setRegister(0x0052, y1); // set vertical start

	lcd_setRegister(0x0051, x2); // set horizontal end
	lcd_setRegister(0x0053, y2); // set vertical end

	LCD_INST_ADD = 0x22;
}

void lcd_setPixel(uint16_t x, uint16_t y, uint16_t color){
	lcd_setArea(x, y, x + 1, y + 1);
	lcd_setRegister(0x22, color);
}
